import { useLogger, defineNuxtModule } from '@nuxt/kit';

const meta = {
  name: "nuxt-font-preload",
  configKey: "fontPreload",
  compatibility: {
    nuxt: "^3.0.0"
  }
};

const logger = useLogger(meta.name);
function generateFontFaces(options) {
  let styleContent = "";
  for (const font of options.fonts) {
    if (!font.weights?.length) {
      logger.warn(`${font.fontFamily} is missing weights array`);
      continue;
    }
    font.weights.forEach((weightConfig) => {
      if (!weightConfig.sources?.length)
        return logger.warn(
          `${font.fontFamily}:${weightConfig.weight} is missing sources array`
        );
      const { weight, sources, ...config } = {
        ...font.defaultOptions,
        ...weightConfig
      };
      const cssSrc = sources.map((src) => `url("${src.url}") format("${src.format}")`).join(",");
      const fontDescriptors = Object.entries(config).map(([key, value]) => `${camelToKebabCase(key)}: ${value};`).join(" ");
      styleContent += ` @font-face {
        font-family: "${font.fontFamily}";
        src: ${cssSrc};
        font-weight: ${weight}; 
        ${fontDescriptors}
      }`;
    });
  }
  return styleContent.replaceAll(/\s+/g, " ").trim();
}
function camelToKebabCase(str) {
  return str.replace(/([a-z0-9])([A-Z])/g, "$1-$2").toLowerCase();
}

function generatePreloadLinks(options) {
  const { fonts } = options;
  const preloadSources = fonts.flatMap((font) => {
    return font.weights?.flatMap((weightConfig) => {
      return weightConfig.sources?.filter((source) => source.preload);
    });
  });
  return preloadSources.map((src) => {
    return {
      rel: "preload",
      as: "font",
      href: src.url,
      type: `font/${src.format}`,
      crossorigin: src.crossorigin || "anonymous"
    };
  });
}

const module = defineNuxtModule({
  meta: {
    name: meta.name,
    configKey: meta.configKey,
    compatibility: { nuxt: meta.compatibility.nuxt }
  },
  defaults: {
    fonts: []
  },
  setup(options, nuxt) {
    const customFonts = options?.fonts || [];
    if (!customFonts?.length)
      return;
    const fontFacesStyle = generateFontFaces({ fonts: customFonts });
    const preloadLinks = generatePreloadLinks({ fonts: customFonts });
    nuxt.options.app.head.style?.push({ innerHTML: fontFacesStyle });
    nuxt.options.app.head.link?.push(
      ...preloadLinks
    );
  }
});

export { module as default };
